# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..base import DipyDiffusionInterface


def test_DipyDiffusionInterface_inputs():
    input_map = dict(
        b0_thres=dict(
            usedefault=True,
        ),
        in_bval=dict(
            extensions=None,
            mandatory=True,
        ),
        in_bvec=dict(
            extensions=None,
            mandatory=True,
        ),
        in_file=dict(
            extensions=None,
            mandatory=True,
        ),
        out_prefix=dict(),
    )
    inputs = DipyDiffusionInterface.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
